from typing import Dict, Pattern, Tuple

from typing_extensions import Final, Literal

GET_ITERATOR_CHUNK_SIZE: Final[int] = ...

MULTI: Literal["multi"] = ...
SINGLE: Literal["single"] = ...
CURSOR: Literal["cursor"] = ...
NO_RESULTS: Literal["no results"] = ...

ORDER_PATTERN: Pattern = ...
ORDER_DIR: Dict[str, Tuple[str, str]] = ...

INNER: Literal["INNER JOIN"] = ...
LOUTER: Literal["LEFT OUTER JOIN"] = ...
